@@ -2,7 +2,7 @@
 #ifndef _GLIBMM_DISPATCHER_H
 #define _GLIBMM_DISPATCHER_H
 
-/* $Id: dispatcher.h,v 1.1.1.1 2003/01/07 16:58:45 murrayc Exp $ */
+/* $Id: dispatcher.h,v 1.2 2004/02/10 14:26:04 murrayc Exp $ */
 
 /* Copyright 2002 The gtkmm Development Team
  *
@@ -76,10 +76,10 @@ public:
   void emit();
   void operator()();
 
-  SigC::Connection connect(const SigC::Slot0<void>& slot);
+  sigc::connection connect(const sigc::slot<void>& slot);
 
 private:
-  SigC::Signal0<void> signal_;
+  sigc::signal<void> signal_;
   DispatchNotifier*   notifier_;
 
   // noncopyable
